@@ -40,6 +40,7 @@
 // functionality!
 
 Components.utils.import("resource://enigmail/enigmailCommon.jsm");
+Components.utils.import("resource://enigmail/enigmailCore.jsm");
 Components.utils.import("resource://enigmail/commonFuncs.jsm");
 Components.utils.import("resource://enigmail/keyManagement.jsm");
 
@@ -139,14 +140,22 @@ const PGP_MIME_ALWAYS   = 2;
 const ENIG_POSSIBLE_PGPMIME = -2081;
 const ENIG_PGP_DESKTOP_ATT  = -2082;
 
-var gUsePGPMimeOptionList = ["usePGPMimeNever", "usePGPMimePossible",
+var gUsePGPMimeOptionList = ["usePGPMimeNever",
+                             "usePGPMimePossible",
                              "usePGPMimeAlways"];
 
-var gEnigRecipientsSelection = ["-",
-                                "perRecipientRules",
-                                "perRecipientRulesAndEmail",
-                                "perEmailAddress",
-                                "askRecipientsAlways"];
+// sending options:
+var gEnigEncryptionModel = ["encryptionModelConvenient",
+                            "encryptionModelManually"];
+var gEnigAcceptedKeys = ["acceptedKeysValid",
+                         "acceptedKeysAll"];
+var gEnigAutoSendEncrypted = ["autoSendEncryptedNever",
+                              "autoSendEncryptedIfKeys"];
+var gEnigConfirmBeforeSending = ["confirmBeforeSendingNever",
+                                 "confirmBeforeSendingAlways",
+                                 "confirmBeforeSendingIfEncrypted",
+                                 "confirmBeforeSendingIfNotEncrypted",
+                                 "confirmBeforeSendingIfRules"];
 
 const ENIG_BUTTON_POS_0           = 1;
 const ENIG_BUTTON_POS_1           = 1 << 8;
@@ -342,9 +351,9 @@ function EnigGetDefaultPref(prefName) {
   DEBUG_LOG("enigmailCommon.js: EnigGetDefaultPref: prefName="+prefName+"\n");
   var prefValue=null;
   try {
-    EnigmailCommon.prefBranch.lockPref(prefName);
+    EnigmailCore.prefBranch.lockPref(prefName);
     prefValue = EnigGetPref(prefName);
-    EnigmailCommon.prefBranch.unlockPref(prefName);
+    EnigmailCore.prefBranch.unlockPref(prefName);
   }
   catch (ex) {}
 
@@ -551,7 +560,7 @@ function EnigRevokeKey(keyId, userId, callbackFunc) {
     return false;
 
   var userDesc="0x"+keyId.substr(-8,8)+" - "+userId;
-  if (!EnigConfirm(EnigGetString("revokeKeyAsk", userDesc), EnigGetString("keyMan.button.revokeKey")))
+  if (!EnigConfirm(EnigGetString("revokeKeyQuestion", userDesc), EnigGetString("keyMan.button.revokeKey")))
       return false;
 
   var tmpDir=EnigGetTempDir();
@@ -616,7 +625,6 @@ function EnigCreateRevokeCert(keyId, userId, callbackFunc) {
   if (!enigmailSvc)
     return -1;
 
-  var errorMsgObj = {};
   EnigmailKeyMgmt.genRevokeCert(window, "0x"+keyId, outFile, "1", "",
     function _revokeCertCb(exitCode, errorMsg) {
       if (exitCode != 0) {
